@@ -17,8 +17,9 @@ conditions.
 #include "nc4dispatch.h"
 #include "ncdispatch.h"
 
-#if 0 /*def USE_PNETCDF*/
-#include <pnetcdf.h>
+#ifdef ENABLE_FILEINFO
+static int nc4_get_att_special(NC_HDF5_FILE_INFO_T*, const char*,
+                               nc_type*, nc_type, size_t*, int*, int, void*);
 #endif
 
 int nc4typelen(nc_type type);
@@ -43,10 +44,11 @@ nc4_get_att(int ncid, NC *nc, int varid, const char *name,
    char norm_name[NC_MAX_NAME + 1];
    int i;
    int retval = NC_NOERR;
+   const char** sp;
 
-   if (attnum)
+   if (attnum) {
       my_attnum = *attnum;
-   assert(nc && NC4_DATA(nc));
+   }
 
    LOG((3, "%s: ncid 0x%x varid %d name %s attnum %d mem_type %d",
 	__func__, ncid, varid, name, my_attnum, mem_type));
@@ -60,10 +62,26 @@ nc4_get_att(int ncid, NC *nc, int varid, const char *name,
    if ((retval = nc4_normalize_name(name, norm_name)))
       BAIL(retval);
 
-   /* Find the attribute, if it exists. If we don't find it, we are
-      major failures. */
-   if ((retval = nc4_find_grp_att(grp, varid, norm_name, my_attnum, &att)))
+#ifdef ENABLE_FILEINFO
+   if(nc->ext_ncid == ncid && varid == NC_GLOBAL) {
+	const char** sp;
+	for(sp = NC_RESERVED_SPECIAL_LIST;*sp;sp++) {
+	    if(strcmp(name,*sp)==0) {
+		return nc4_get_att_special(h5, norm_name, xtype, mem_type, lenp, attnum, is_long, data);
+	    }
+	}
+    }
+#endif
+
+   /* Find the attribute, if it exists.
+      <strike>If we don't find it, we are major failures.</strike>
+   */
+   if ((retval = nc4_find_grp_att(grp, varid, norm_name, my_attnum, &att))) {
+     if(retval == NC_ENOTATT)
+	return retval;
+     else
       BAIL(retval);
+   }
 
    /* If mem_type is NC_NAT, it means we want to use the attribute's
     * file type as the mem type as well. */
@@ -84,8 +102,9 @@ nc4_get_att(int ncid, NC *nc, int varid, const char *name,
       *lenp = att->len;
    if (xtype)
       *xtype = att->nc_typeid;
-   if (attnum)
+   if (attnum) {
       *attnum = att->attnum;
+   }
 
    /* Zero len attributes are easy to read! */
    if (!att->len)
@@ -232,23 +251,35 @@ nc4_put_att(int ncid, NC *nc, int varid, const char *name,
    if ((retval = nc4_check_name(name, norm_name)))
       return retval;
 
+#ifdef ENABLE_FILEINFO
+   if(nc->ext_ncid == ncid && varid == NC_GLOBAL) {
+	const char** sp;
+	for(sp = NC_RESERVED_SPECIAL_LIST;*sp;sp++) {
+	    if(strcmp(name,*sp)==0) {
+		return NC_ENOTATT; /* Not settable */
+	    }
+	}
+    }
+#endif
+
    /* Find att, if it exists. */
    if (varid == NC_GLOBAL)
       attlist = &grp->att;
    else
    {
-      for (var = grp->var; var; var = var->l.next)
-	 if (var->varid == varid)
-	 {
-	    attlist = &var->att;
-	    break;
-	 }
-      if (!var)
-	 return NC_ENOTVAR;
+     for (var = grp->var; var; var = var->l.next)
+       if (var->varid == varid)
+         {
+           attlist = &var->att;
+           break;
+         }
+     if (!var)
+       return NC_ENOTVAR;
    }
+
    for (att = *attlist; att; att = att->l.next)
-      if (!strcmp(att->name, norm_name))
-	 break;
+     if (!strcmp(att->name, norm_name))
+       break;
 
    if (!att)
    {
@@ -266,14 +297,14 @@ nc4_put_att(int ncid, NC *nc, int varid, const char *name,
    {
       /* For an existing att, if we're not in define mode, the len
 	 must not be greater than the existing len for classic model. */
-      if (!(h5->flags & NC_INDEF) &&
-	  len * nc4typelen(file_type) > (size_t)att->len * nc4typelen(att->nc_typeid))
-      {
-	 if (h5->cmode & NC_CLASSIC_MODEL)
-	    return NC_EINDEFINE;
-	 if ((retval = NC4_redef(ncid)))
-	    BAIL(retval);
-      }
+     if (!(h5->flags & NC_INDEF) &&
+         len * nc4typelen(file_type) > (size_t)att->len * nc4typelen(att->nc_typeid))
+       {
+         if (h5->cmode & NC_CLASSIC_MODEL)
+           return NC_EINDEFINE;
+         if ((retval = NC4_redef(ncid)))
+           BAIL(retval);
+       }
    }
 
    /* We must have two valid types to continue. */
@@ -301,9 +332,9 @@ nc4_put_att(int ncid, NC *nc, int varid, const char *name,
    {
       LOG((3, "adding attribute %s to the list...", norm_name));
       if ((res = nc4_att_list_add(attlist, &att)))
-	 BAIL (res);
+        BAIL (res);
       if (!(att->name = strdup(norm_name)))
-         return NC_ENOMEM;
+        return NC_ENOMEM;
    }
 
    /* Now fill in the metadata. */
@@ -344,72 +375,72 @@ nc4_put_att(int ncid, NC *nc, int varid, const char *name,
 
       /* Fill value must be same type and have exactly one value */
       if (att->nc_typeid != var->type_info->nc_typeid)
-	 return NC_EBADTYPE;
+        return NC_EBADTYPE;
       if (att->len != 1)
-	 return NC_EINVAL;
+        return NC_EINVAL;
 
       /* If we already wrote to the dataset, then return an error. */
       if (var->written_to)
-	 return NC_ELATEFILL;
+        return NC_ELATEFILL;
 
       /* If fill value hasn't been set, allocate space. Of course,
        * vlens have to be different... */
       if ((retval = nc4_get_typelen_mem(grp->nc4_info, var->type_info->nc_typeid, 0,
-					&type_size)))
-	 return retval;
+                                        &type_size)))
+        return retval;
 
       /* Already set a fill value? Now I'll have to free the old
        * one. Make up your damn mind, would you? */
       if (var->fill_value)
-      {
-         if (var->type_info->nc_type_class == NC_VLEN)
-         {
-            if ((retval = nc_free_vlen(var->fill_value)))
-               return retval;
-         }
-         else if (var->type_info->nc_type_class == NC_STRING)
-         {
-            if (*(char **)var->fill_value)
-               free(*(char **)var->fill_value);
-         }
-	 free(var->fill_value);
-      }
+        {
+          if (var->type_info->nc_type_class == NC_VLEN)
+            {
+              if ((retval = nc_free_vlen(var->fill_value)))
+                return retval;
+            }
+          else if (var->type_info->nc_type_class == NC_STRING)
+            {
+              if (*(char **)var->fill_value)
+                free(*(char **)var->fill_value);
+            }
+          free(var->fill_value);
+        }
 
       /* Allocate space for the fill value. */
       if (var->type_info->nc_type_class == NC_VLEN)
-	 size = sizeof(hvl_t);
+        size = sizeof(hvl_t);
       else if (var->type_info->nc_type_class == NC_STRING)
-	 size = sizeof(char *);
+        size = sizeof(char *);
       else
-	 size = type_size;
+        size = type_size;
 
       if (!(var->fill_value = calloc(1, size)))
-	 return NC_ENOMEM;
+        return NC_ENOMEM;
 
       /* Copy the fill_value. */
       LOG((4, "Copying fill value into metadata for variable %s", var->name));
       if (var->type_info->nc_type_class == NC_VLEN)
-      {
-	 nc_vlen_t *in_vlen = (nc_vlen_t *)data, *fv_vlen = (nc_vlen_t *)(var->fill_value);
+        {
+          nc_vlen_t *in_vlen = (nc_vlen_t *)data, *fv_vlen = (nc_vlen_t *)(var->fill_value);
 
-	 fv_vlen->len = in_vlen->len;
-	 if (!(fv_vlen->p = malloc(size * in_vlen->len)))
-	    return NC_ENOMEM;
-	 memcpy(fv_vlen->p, in_vlen->p, in_vlen->len * size);
-      }
+          fv_vlen->len = in_vlen->len;
+          if (!(fv_vlen->p = malloc(size * in_vlen->len)))
+            return NC_ENOMEM;
+          memcpy(fv_vlen->p, in_vlen->p, in_vlen->len * size);
+        }
       else if (var->type_info->nc_type_class == NC_STRING)
-      {
-         if(NULL != (*(char **)data))
-         {
-            if (!(*(char **)(var->fill_value) = malloc(strlen(*(char **)data) + 1)))
-               return NC_ENOMEM;
-            strcpy(*(char **)var->fill_value, *(char **)data);
-         }
-         else
+        {
+          if(NULL != (*(char **)data))
+            {
+              if (!(*(char **)(var->fill_value) = malloc(strlen(*(char **)data) + 1)))
+                return NC_ENOMEM;
+              strcpy(*(char **)var->fill_value, *(char **)data);
+            }
+          else
             *(char **)var->fill_value = NULL;
-      }
+        }
       else
-	 memcpy(var->fill_value, data, type_size);
+        memcpy(var->fill_value, data, type_size);
 
       /* Indicate that the fill value was changed, if the variable has already
        * been created in the file, so the dataset gets deleted and re-created. */
@@ -518,7 +549,7 @@ nc4_put_att(int ncid, NC *nc, int varid, const char *name,
 }
 
 /* Learn about an att. All the nc4 nc_inq_ functions just call
- * add_meta_get to get the metadata on an attribute. */
+ * nc4_get_att to get the metadata on an attribute. */
 int
 NC4_inq_att(int ncid, int varid, const char *name, nc_type *xtypep, size_t *lenp)
 {
@@ -535,21 +566,8 @@ NC4_inq_att(int ncid, int varid, const char *name, nc_type *xtypep, size_t *lenp
    h5 = NC4_DATA(nc);
    assert(h5);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-   {
-      MPI_Offset mpi_len;
-      int ret = ncmpi_inq_att(nc->int_ncid, varid, name, xtypep, &mpi_len);
-      if (ret != NC_NOERR)
-	 return ret;
-      if (lenp)
-	 *lenp = mpi_len;
-   }
-#endif /* USE_PNETCDF */
-
    /* Handle netcdf-4 files. */
-   return nc4_get_att(ncid, nc, varid, name, xtypep, NC_UBYTE, lenp, NULL, 0, NULL);
+   return nc4_get_att(ncid, nc, varid, name, xtypep, NC_NAT, lenp, NULL, 0, NULL);
 }
 
 /* Learn an attnum, given a name. */
@@ -558,6 +576,7 @@ NC4_inq_attid(int ncid, int varid, const char *name, int *attnump)
 {
    NC *nc;
    NC_HDF5_FILE_INFO_T *h5;
+   int stat;
 
    LOG((2, "nc_inq_attid: ncid 0x%x varid %d name %s", ncid, varid, name));
 
@@ -569,15 +588,10 @@ NC4_inq_attid(int ncid, int varid, const char *name, int *attnump)
    h5 = NC4_DATA(nc);
    assert(h5);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-      return ncmpi_inq_attid(nc->int_ncid, varid, name, attnump);
-#endif /* USE_PNETCDF */
-
    /* Handle netcdf-4 files. */
-   return nc4_get_att(ncid, nc, varid, name, NULL, NC_UBYTE,
+   stat = nc4_get_att(ncid, nc, varid, name, NULL, NC_NAT,
 		      NULL, attnump, 0, NULL);
+   return stat;
 }
 
 
@@ -601,12 +615,6 @@ NC4_inq_attname(int ncid, int varid, int attnum, char *name)
    h5 = NC4_DATA(nc);
    assert(h5);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-      return ncmpi_inq_attname(nc->int_ncid, varid, attnum, name);
-#endif /* USE_PNETCDF */
-
    /* Handle netcdf-4 files. */
    if ((retval = nc4_find_nc_att(ncid, varid, NULL, attnum, &att)))
       return retval;
@@ -653,12 +661,6 @@ NC4_rename_att(int ncid, int varid, const char *name,
    if (h5->no_write)
      return NC_EPERM;
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-      return ncmpi_rename_att(nc->int_ncid, varid, name, newname);
-#endif /* USE_PNETCDF */
-
    /* Check and normalize the name. */
    if ((retval = nc4_check_name(newname, norm_newname)))
       return retval;
@@ -763,12 +765,6 @@ NC4_del_att(int ncid, int varid, const char *name)
    if (h5->no_write)
       return NC_EPERM;
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-      return ncmpi_del_att(nc->int_ncid, varid, name);
-#endif /* USE_PNETCDF */
-
    /* If it's not in define mode, forget it. */
    if (!(h5->flags & NC_INDEF))
    {
@@ -860,49 +856,74 @@ nc4_put_att_tc(int ncid, int varid, const char *name, nc_type file_type,
    h5 = NC4_DATA(nc);
    assert(h5);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-   {
-      if (mem_type == NC_UBYTE)
-	 mem_type = NC_BYTE;
+   if(nc->ext_ncid == ncid && varid == NC_GLOBAL) {
+      const char** reserved = NC_RESERVED_ATT_LIST;
+      for(;*reserved;reserved++) {
+	if(strcmp(name,*reserved)==0)
+	    return NC_ENAMEINUSE;
+      }
+   }
 
-      switch(mem_type)
-      {
-	 case NC_BYTE:
-	    return ncmpi_put_att_schar(nc->int_ncid, varid, name,
-				     file_type, len, op);
-	 case NC_CHAR:
-	    return ncmpi_put_att_text(nc->int_ncid, varid, name,
-				    len, op);
-	 case NC_SHORT:
-	    return ncmpi_put_att_short(nc->int_ncid, varid, name,
-				     file_type, len, op);
-	 case NC_INT:
-	    if (mem_type_is_long)
-	       return ncmpi_put_att_long(nc->int_ncid, varid, name,
-				       file_type, len, op);
-	    else
-	       return ncmpi_put_att_int(nc->int_ncid, varid, name,
-				      file_type, len, op);
-	 case NC_FLOAT:
-	    return ncmpi_put_att_float(nc->int_ncid, varid, name,
-				     file_type, len, op);
-	 case NC_DOUBLE:
-	    return ncmpi_put_att_double(nc->int_ncid, varid, name,
-				      file_type, len, op);
-	 case NC_NAT:
-	 default:
-	    return NC_EBADTYPE;
+   if(varid != NC_GLOBAL) {
+      const char** reserved = NC_RESERVED_VARATT_LIST;
+      for(;*reserved;reserved++) {
+	if(strcmp(name,*reserved)==0)
+	    return NC_ENAMEINUSE;
       }
    }
-#endif /* USE_PNETCDF */
 
    /* Otherwise, handle things the netcdf-4 way. */
    return nc4_put_att(ncid, nc, varid, name, file_type, mem_type, len,
 		      mem_type_is_long, op);
 }
 
+#ifdef ENABLE_FILEINFO
+static int
+nc4_get_att_special(NC_HDF5_FILE_INFO_T* h5, const char* name,
+                    nc_type* filetypep, nc_type mem_type, size_t* lenp,
+                    int* attnump, int is_long, void* data)
+{
+    /* Fail if asking for att id */
+    if(attnump)
+	return NC_EATTMETA;
+
+    if(strcmp(name,NCPROPS)==0) {
+	if(h5->fileinfo->propattr.version == 0)
+	    return NC_ENOTATT;
+	if(mem_type == NC_NAT) mem_type = NC_CHAR;
+	if(mem_type != NC_CHAR)
+	    return NC_ECHAR;
+	if(filetypep) *filetypep = NC_CHAR;
+	if(lenp) *lenp = strlen(h5->fileinfo->propattr.text);
+	if(data) strcpy((char*)data,h5->fileinfo->propattr.text);
+    } else if(strcmp(name,ISNETCDF4ATT)==0
+              || strcmp(name,SUPERBLOCKATT)==0) {
+	unsigned long long iv = 0;
+	if(filetypep) *filetypep = NC_INT;
+	if(lenp) *lenp = 1;
+	if(strcmp(name,SUPERBLOCKATT)==0)
+	    iv = (unsigned long long)h5->fileinfo->superblockversion;
+	else /* strcmp(name,ISNETCDF4ATT)==0 */
+	    iv = NC4_isnetcdf4(h5);
+	if(mem_type == NC_NAT) mem_type = NC_INT;
+	if(data)
+	switch (mem_type) {
+	case NC_BYTE: *((char*)data) = (char)iv; break;
+	case NC_SHORT: *((short*)data) = (short)iv; break;
+	case NC_INT: *((int*)data) = (int)iv; break;
+	case NC_UBYTE: *((unsigned char*)data) = (unsigned char)iv; break;
+	case NC_USHORT: *((unsigned short*)data) = (unsigned short)iv; break;
+	case NC_UINT: *((unsigned int*)data) = (unsigned int)iv; break;
+	case NC_INT64: *((long long*)data) = (long long)iv; break;
+	case NC_UINT64: *((unsigned long long*)data) = (unsigned long long)iv; break;
+	default:
+	    return NC_ERANGE;
+	}
+    }
+    return NC_NOERR;
+}
+#endif
+
 /* Read an attribute of any type, with type conversion. This may be
  * called by any of the nc_get_att_* functions. */
 int
@@ -923,37 +944,6 @@ nc4_get_att_tc(int ncid, int varid, const char *name, nc_type mem_type,
    h5 = NC4_DATA(nc);
    assert(h5);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-   {
-      if (mem_type == NC_UBYTE)
-	 mem_type = NC_BYTE;
-
-      switch(mem_type)
-      {
-	 case NC_BYTE:
-	    return ncmpi_get_att_schar(nc->int_ncid, varid, name, ip);
-	 case NC_CHAR:
-	    return ncmpi_get_att_text(nc->int_ncid, varid, name, ip);
-	 case NC_SHORT:
-	    return ncmpi_get_att_short(nc->int_ncid, varid, name, ip);
-	 case NC_INT:
-	    if (mem_type_is_long)
-	       return ncmpi_get_att_long(nc->int_ncid, varid, name, ip);
-	    else
-	       return ncmpi_get_att_int(nc->int_ncid, varid, name, ip);
-	 case NC_FLOAT:
-	    return ncmpi_get_att_float(nc->int_ncid, varid, name, ip);
-	 case NC_DOUBLE:
-	    return ncmpi_get_att_double(nc->int_ncid, varid, name, ip);
-	 case NC_NAT:
-	 default:
-	    return NC_EBADTYPE;
-      }
-   }
-#endif /* USE_PNETCDF */
-
    return nc4_get_att(ncid, nc, varid, name, NULL, mem_type,
 		      NULL, NULL, mem_type_is_long, ip);
 }
